#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <ctype.h>

#if 0
#    define debug_printf printf
#else
#    define debug_printf(args...)
#endif

#ifndef GCC_WRAPPER_OPTS
#    define GCC_WRAPPER_OPTS ""
#endif

char *options[]={GCC_WRAPPER_OPTS, "-O2", "-pipe", NULL};

/*
 *  Check if 'x' is an -Ox option.
 *
 *  Returncodes:  0 = not a -Ox option
 *                1 = -O0
 *                2 = -O1 or -O
 *                3 = -O2
 *                ....
 */
int is_o_opt(char * x) {
	if (x[0]=='-' && x[1]=='O' &&
	    (!x[2] || (isdigit(x[2]) && !x[3])) ) {
		return x[2] ? (x[2]-'0')+1 : 2;
	}
	return 0;
}

/*
 *  Main function.
 */
int main(int argc, char ** argv) {
	char * prognam;
	char progpath[100];
	char **newargv;
	int c1,c2,c3,useit;
	char *delim;
	
	/* Make sure that newargv is big enough */
	for (c1=0; options[c1]; c1++) ;
	newargv=malloc( sizeof(char*) * (argc+c1+10) );
	if (newargv == NULL) {
		printf("Can't malloc char *newargv[%d].\n",argc+c1+10);
		return 250;
	}
	
	/* Only one -Ox option is allowed in options[] */
	for (c1=c2=0; options[c1]; c1++) {
		if (is_o_opt(options[c1])) {
			if (c2 == 0) c2=1;
			else options[c1]="";
		}
	}
	
	/* Get the program name (without path) */
	prognam=rindex(argv[0],'/');
	if (prognam) prognam++; else prognam=argv[0];
	
	/* Get the full path name of the real program */
	sprintf(progpath,"/bin/%s",prognam);
	if (access(progpath,F_OK)) {
	     sprintf(progpath,"/usr/bin/%s",prognam);
 	    if (access(progpath,F_OK)) {
		sprintf(progpath,"/usr/local/bin/%s",prognam);
		if (access(progpath,F_OK)) {
		    printf("Can't find full path name for '%s' "
		           "-> abort.\n",prognam);
		    return 250;
		}
	    }
	}
	
	newargv[0]=argv[0];
	for (c1=c2=1; argv[c2]; c2++) {
		useit=1;
		for (c3=0; options[c3]; c3++) {
			
			/* options[c3] is disabled */
			if (!options[c3][0]) continue;
			
			/* It's the same */
			if (!strcmp(argv[c2],options[c3])) { useit=0; break; }
			
			/* It's the same -foo=bar option */
			if ( (delim=index(options[c3],'=')) != NULL &&
			     !strncmp(argv[c2],options[c3],delim-options[c3]) )
				{ useit=0; break; }
			
			/* It's a lower -O option than the one stored in */
			/* options[]. If it is higher, the option in the */
			/* options[] array needs to be disabled. */
			if ( is_o_opt(argv[c2]) && is_o_opt(options[c3]) ) {
				if ( is_o_opt(argv[c2]) < is_o_opt(options[c3]))
					{ useit=0; break; }
				else
					{ options[c3]=""; }
			}
		}
		if (useit) {
			debug_printf("New Parameter #%d: %s\n",c1,argv[c2]);
			newargv[c1++]=argv[c2];
		}
	}
	for (c3=0; options[c3]; c3++) {
		if (options[c3][0]) {
			debug_printf("New Parameter #%d: %s\n",c1,options[c3]);
			newargv[c1++]=options[c3];
		}
	}
	
	newargv[c1]=NULL;
	execv(progpath,newargv);
	
	printf("Can't execute '%s' -> abort.\n",progpath);
	return 250;
}
